@@ -36,6 +36,7 @@ import org.apache.spark.unsafe.types.UTF8String
 import org.geotools.geometry.jts.JTS
 import org.geotools.referencing.CRS
 import org.locationtech.jts.algorithm.MinimumBoundingCircle
+import org.locationtech.jts.geom.util.GeometryFixer
 import org.locationtech.jts.geom.{PrecisionModel, _}
 import org.locationtech.jts.io.{ByteOrderValues, WKBWriter, WKTWriter}
 import org.locationtech.jts.linearref.LengthIndexedLine
@@ -48,6 +49,8 @@ import org.locationtech.jts.precision.GeometryPrecisionReducer
 import org.locationtech.jts.simplify.TopologyPreservingSimplifier
 import org.opengis.referencing.operation.MathTransform
 import org.wololo.jts2geojson.GeoJSONWriter
+import org.locationtech.jts.geom.Geometry
+import org.locationtech.jts.geom.Coordinate
 
 import java.nio.ByteOrder
 import scala.util.{Failure, Success, Try}
@@ -76,6 +79,50 @@ case class ST_Distance(inputExpressions: Seq[Expression])
   }
 }
 
+
+case class ST_YMax(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    val seqRev : Array[Coordinate] = geometry.getCoordinates()
+    var maxVal:Double = Double.MinValue
+    for(x <- seqRev ){
+      maxVal=Math.max(maxVal,x.getY())
+    }
+    maxVal
+  }
+
+  override def dataType: DataType = DoubleType
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+case class ST_YMin(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    val seqRev : Array[Coordinate] = geometry.getCoordinates()
+    var minVal:Double = Double.MaxValue
+    for(x <- seqRev){
+      minVal=Math.min(minVal,x.getY())
+    }
+    minVal
+  }
+
+  override def dataType: DataType = DoubleType
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
 case class ST_3DDistance(inputExpressions: Seq[Expression])
   extends BinaryGeometryExpression with CodegenFallback {
   assert(inputExpressions.length == 2)
@@ -343,49 +390,38 @@ case class ST_Intersection(inputExpressions: Seq[Expression])
 }
 
 /**
-  * Given an invalid polygon or multipolygon and removeHoles boolean flag, create a valid representation of the geometry
+  * Given an invalid geometry, create a valid representation of the geometry.
+  * See: http://lin-ear-th-inking.blogspot.com/2021/05/fixing-invalid-geometry-with-jts.html
   *
   * @param inputExpressions
   */
 case class ST_MakeValid(inputExpressions: Seq[Expression])
-  extends Generator with CodegenFallback with UserDataGeneratator {
-  assert(inputExpressions.length == 2)
+  extends Expression with CodegenFallback {
+  assert(inputExpressions.length == 1 || inputExpressions.length == 2)
 
-  override def elementSchema: StructType = new StructType().add("Geometry", new GeometryUDT)
-
-  override def toString: String = s" **${ST_MakeValid.getClass.getName}** "
-
-  override def eval(input: InternalRow): TraversableOnce[InternalRow] = {
-    val geometry = GeometrySerializer.deserialize(inputExpressions(0).eval(input).asInstanceOf[ArrayData])
-    val removeHoles = inputExpressions(1).eval(input).asInstanceOf[Boolean]
-
-    // in order to do flatMap on java collections(util.List[Polygon])
-    import scala.jdk.CollectionConverters._
-
-    // makeValid works only on polygon or multipolygon
-    if (!geometry.getGeometryType.equalsIgnoreCase("POLYGON") && !geometry.getGeometryType.equalsIgnoreCase("MULTIPOLYGON")) {
-      throw new IllegalArgumentException("ST_MakeValid works only on Polygons and MultiPolygons")
+  override def eval(input: InternalRow): Any = {
+    val geometry = inputExpressions.head.toGeometry(input)
+    val keepCollapsed = if (inputExpressions.length == 2) {
+      inputExpressions(1).eval(input).asInstanceOf[Boolean]
+    } else {
+      false
     }
-
-    val validGeometry = geometry match {
-      case g: MultiPolygon =>
-        (0 until g.getNumGeometries).flatMap(i => {
-          val polygon = g.getGeometryN(i).asInstanceOf[Polygon]
-          JTS.makeValid(polygon, removeHoles).asScala.iterator
-        })
-      case g: Polygon =>
-        JTS.makeValid(g, removeHoles).asScala.iterator
-      case _ => Nil
+    (geometry) match {
+      case (geometry: Geometry) => nullSafeEval(geometry, keepCollapsed)
+      case _ => null
     }
-
-    val result = validGeometry.map(g => {
-      val serializedGeometry = GeometrySerializer.serialize(g.asInstanceOf[Geometry])
-      InternalRow(new GenericArrayData(serializedGeometry))
-    })
-
-    result
   }
 
+  private def nullSafeEval(geometry: Geometry, keepCollapsed: Boolean) = {
+    val fixer = new GeometryFixer(geometry)
+    fixer.setKeepCollapsed(keepCollapsed)
+    fixer.getResult.toGenericArrayData
+  }
+
+  override def nullable: Boolean = true
+
+  override def dataType: DataType = GeometryUDT
+
   override def children: Seq[Expression] = inputExpressions
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
@@ -1537,4 +1573,241 @@ case class ST_Multi(inputExpressions: Seq[Expression]) extends UnaryGeometryExpr
   override protected def nullSafeEval(geometry: Geometry): Any ={
     Collect.createMultiGeometry(Seq(geometry)).toGenericArrayData
   }
+}
+
+/**
+ * Returns a POINT guaranteed to lie on the surface.
+ *
+ * @param inputExpressions Geometry
+ */
+case class ST_PointOnSurface(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    new GenericArrayData(GeometrySerializer.serialize(GeomUtils.getInteriorPoint(geometry)))
+  }
+
+  override def dataType: DataType = GeometryUDT
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+/**
+ * Returns the geometry with vertex order reversed
+ *
+ * @param inputExpressions
+ */
+case class ST_Reverse(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    new GenericArrayData(GeometrySerializer.serialize(geometry.reverse()))
+  }
+
+  override def dataType: DataType = GeometryUDT
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+/**
+ * Returns the nth point in the geometry, provided it is a linestring
+ *
+ * @param inputExpressions sequence of 2 input arguments, a geometry and a value 'n'
+ */
+case class ST_PointN(inputExpressions: Seq[Expression])
+  extends Expression with CodegenFallback {
+  inputExpressions.validateLength(2)
+
+  override def nullable: Boolean = true
+
+  override def eval(input: InternalRow): Any = {
+    val geometry = inputExpressions.head.toGeometry(input)
+    val n = inputExpressions(1).toInt(input)
+    getNthPoint(geometry, n)
+  }
+
+  private def getNthPoint(geometry: Geometry, n: Int): GenericArrayData = {
+    geometry match {
+      case linestring: LineString => val point = GeomUtils.getNthPoint(linestring, n)
+        point match {
+          case geometry: Geometry => new GenericArrayData(GeometrySerializer.serialize(geometry))
+          case _ => null
+        }
+      case _ => null
+    }
+  }
+  override def dataType: DataType = GeometryUDT
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+      copy(inputExpressions = newChildren)
+  }
+}
+
+ /*
+ * Forces the geometries into a "2-dimensional mode" so that all output representations will only have the X and Y coordinates.
+ *
+ * @param inputExpressions
+ */
+case class ST_Force_2D(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    new GenericArrayData(GeometrySerializer.serialize(GeomUtils.get2dGeom(geometry)))
+  }
+
+  override def dataType: DataType = GeometryUDT
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+/**
+ * Returns the geometry in EWKT format
+ *
+ * @param inputExpressions
+ */
+case class ST_AsEWKT(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    UTF8String.fromString(GeomUtils.getEWKT(geometry))
+  }
+
+  override def dataType: DataType = StringType
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+/**
+ * Test if Geometry is empty geometry.
+ *
+ * @param inputExpressions
+ */
+case class ST_IsEmpty(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    geometry.isEmpty()
+  }
+
+  override def dataType: DataType = BooleanType
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+/**
+ * Test if returning Max X coordinate value.
+ *
+ * @param inputExpressions
+ */
+case class ST_XMax(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    var coord:Array[Coordinate] = geometry.getCoordinates()
+    var maxval = Double.MinValue
+    for (point<-coord) {
+      if(point.getX()>maxval){
+        maxval = point.getX()
+      }
+    }
+    maxval
+
+  }
+
+  override def dataType: DataType = DoubleType
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+/**
+ * Test if returning Min X coordinate value.
+ *
+ * @param inputExpressions
+ */
+case class ST_XMin(inputExpressions: Seq[Expression])
+  extends UnaryGeometryExpression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+
+  override protected def nullSafeEval(geometry: Geometry): Any = {
+    var coord: Array[Coordinate] = geometry.getCoordinates()
+    var minval = Double.MaxValue
+    for (point <- coord) {
+      if (point.getX() < minval) {
+        minval = point.getX()
+      }
+    }
+    minval
+
+  }
+
+  override def dataType: DataType = DoubleType
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+
+/**
+ * Returns the areal geometry formed by the constituent linework of the input geometry assuming all inner geometries represent holes
+ *
+ * @param inputExpressions
+ */
+case class ST_BuildArea(inputExpressions: Seq[Expression])
+  extends Expression with CodegenFallback {
+  assert(inputExpressions.length == 1)
+
+  override def nullable: Boolean = true
+
+  override def eval(input: InternalRow): Any = {
+    val geometry = inputExpressions.head.toGeometry(input)
+    geometry match {
+      case geom: Geometry => new GenericArrayData(GeometrySerializer.serialize(GeomUtils.buildArea(geom)))
+      case _ => null
+    }
+  }
+
+  override def dataType: DataType = GeometryUDT
+
+  override def children: Seq[Expression] = inputExpressions
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]): Expression = {
+    copy(inputExpressions = newChildren)
+  }
 }
\ No newline at end of file
