@@ -18,7 +18,7 @@
  */
 package org.apache.spark.sql.sedona_sql.strategy.join
 
-import scala.collection.JavaConverters._
+import scala.jdk.CollectionConverters._
 
 import org.apache.sedona.core.enums.IndexType
 import org.apache.spark.broadcast.Broadcast
@@ -26,15 +26,15 @@ import org.apache.spark.internal.Logging
 import org.apache.spark.rdd.RDD
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.{Attribute, BindReferences, Expression, UnsafeRow}
-import org.apache.spark.sql.execution.{SparkPlan, UnaryExecNode}
-
+import org.apache.spark.sql.execution.SparkPlan
+import org.apache.spark.sql.sedona_sql.execution.SedonaUnaryExecNode
 
 
 case class SpatialIndexExec(child: SparkPlan,
                             shape: Expression,
                             indexType: IndexType,
                             radius: Option[Expression] = None)
-  extends UnaryExecNode
+  extends SedonaUnaryExecNode
     with TraitJoinQueryBase
     with Logging {
 
@@ -58,4 +58,8 @@ case class SpatialIndexExec(child: SparkPlan,
     spatialRDD.buildIndex(indexType, false)
     sparkContext.broadcast(spatialRDD.indexedRawRDD.take(1).asScala.head).asInstanceOf[Broadcast[T]]
   }
+
+  protected def withNewChildInternal(newChild: SparkPlan): SparkPlan = {
+    copy(child = newChild)
+  }
 }
