@@ -22,6 +22,7 @@ package org.apache.sedona.sql
 import org.apache.commons.codec.binary.Hex
 import org.apache.sedona.sql.implicits._
 import org.apache.spark.sql.functions._
+import org.apache.spark.sql.sedona_sql.expressions.ST_MakeValid
 import org.apache.spark.sql.{DataFrame, Row}
 import org.geotools.geometry.jts.WKTReader2
 import org.locationtech.jts.algorithm.MinimumBoundingCircle
@@ -64,6 +65,16 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
       assert(functionDf.count() > 0);
     }
 
+    it("Passed ST_YMax") {
+      var test = sparkSession.sql("SELECT ST_YMax(ST_GeomFromWKT('POLYGON ((-3 -3, 3 -3, 3 3, -3 3, -3 -3))'))")
+      assert(test.take(1)(0).get(0).asInstanceOf[Double] == 3.0)
+    }
+
+    it("Passed ST_YMin") {
+      var test = sparkSession.sql("SELECT ST_YMin(ST_GeomFromWKT('POLYGON ((-3 -3, 3 -3, 3 3, -3 3, -3 -3))'))")
+      assert(test.take(1)(0).get(0).asInstanceOf[Double] == -3.0)
+    }
+
     it("Passed ST_Centroid") {
       var polygonWktDf = sparkSession.read.format("csv").option("delimiter", "\t").option("header", "false").load(mixedWktGeometryInputLocation)
       polygonWktDf.createOrReplaceTempView("polygontable")
@@ -203,62 +214,42 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
     it("Passed ST_MakeValid On Invalid Polygon") {
 
       val df = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((1 5, 1 1, 3 3, 5 3, 7 1, 7 5, 5 3, 3 3, 1 5))') AS polygon")
-      df.createOrReplaceTempView("table")
 
-      val result = sparkSession.sql(
-        """
-          |SELECT geometryValid.polygon
-          |FROM table
-          |LATERAL VIEW ST_MakeValid(polygon, false) geometryValid AS polygon
-          |""".stripMargin
-      ).collect()
+      val result = df.withColumn("polygon", expr("ST_MakeValid(polygon)")).collect()
 
-      val wktReader = new WKTReader2()
-      val firstValidGeometry = wktReader.read("POLYGON ((1 5, 3 3, 1 1, 1 5))")
-      val secondValidGeometry = wktReader.read("POLYGON ((5 3, 7 5, 7 1, 5 3))")
-
-      assert(result.exists(row => row.getAs[Geometry](0).equals(firstValidGeometry)))
-      assert(result.exists(row => row.getAs[Geometry](0).equals(secondValidGeometry)))
+      assert(result.length == 1)
+      assert(result.take(1)(0).get(0).asInstanceOf[Geometry].toText() == "MULTIPOLYGON (((1 5, 3 3, 1 1, 1 5)), ((5 3, 7 5, 7 1, 5 3)))")
     }
 
     it("Passed ST_MakeValid On Invalid MultiPolygon") {
 
       val df = sparkSession.sql("SELECT ST_GeomFromWKT('MULTIPOLYGON(((0 0, 3 0, 3 3, 0 3, 0 0)), ((3 0, 6 0, 6 3, 3 3, 3 0)))') AS multipolygon")
-      df.createOrReplaceTempView("table")
 
-      val result = sparkSession.sql(
-        """
-          |SELECT geometryValid.multipolygon
-          |FROM table
-          |LATERAL VIEW ST_MakeValid(multipolygon, false) geometryValid AS multipolygon
-          |""".stripMargin
-      ).collect()
+      val result = df.withColumn("multipolygon", expr("ST_MakeValid(multipolygon)")).collect()
 
-      val wktReader = new WKTReader2()
-      val firstValidGeometry = wktReader.read("POLYGON ((0 3, 3 3, 3 0, 0 0, 0 3))")
-      val secondValidGeometry = wktReader.read("POLYGON ((3 3, 6 3, 6 0, 3 0, 3 3))")
-
-      assert(result.exists(row => row.getAs[Geometry](0).equals(firstValidGeometry)))
-      assert(result.exists(row => row.getAs[Geometry](0).equals(secondValidGeometry)))
+      assert(result.length == 1)
+      assert(result.take(1)(0).get(0).asInstanceOf[Geometry].toText() == "POLYGON ((0 3, 3 3, 6 3, 6 0, 3 0, 0 0, 0 3))")
     }
 
     it("Passed ST_MakeValid On Valid Polygon") {
 
       val df = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((1 1, 8 1, 8 8, 1 8, 1 1))') AS polygon")
-      df.createOrReplaceTempView("table")
 
-      val result = sparkSession.sql(
-        """
-          |SELECT geometryValid.polygon
-          |FROM table
-          |LATERAL VIEW ST_MakeValid(polygon, false) geometryValid AS polygon
-          |""".stripMargin
-      ).collect()
+      val result = df.withColumn("polygon", expr("ST_MakeValid(polygon)")).collect()
 
-      val wktReader = new WKTReader2()
-      val validGeometry = wktReader.read("POLYGON((1 1, 8 1, 8 8, 1 8, 1 1))")
+      assert(result.length == 1)
+      assert(result.take(1)(0).get(0).asInstanceOf[Geometry].toText() == "POLYGON ((1 1, 1 8, 8 8, 8 1, 1 1))")
+    }
 
-      assert(result.exists(row => row.getAs[Geometry](0).equals(validGeometry)))
+    it("Passed ST_MakeValid on Invalid LineString") {
+
+      val df = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(1 1, 1 1)') AS geom")
+
+      val result = df.selectExpr("ST_MakeValid(geom)", "ST_MakeValid(geom, true)").collect()
+
+      assert(result.length == 1)
+      assert(result.take(1)(0).get(0).asInstanceOf[Geometry].toText() == "LINESTRING EMPTY")
+      assert(result.take(1)(0).get(1).asInstanceOf[Geometry].toText() == "POINT (1 1)")
     }
 
     it("Passed ST_SimplifyPreserveTopology") {
@@ -1285,6 +1276,189 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
 
   }
 
+
+
+  it ("Should pass ST_PointOnSurface") {
+
+    val geomTestCases1 = Map(
+      "'POINT(0 5)'"
+        -> "POINT (0 5)",
+      "'LINESTRING(0 5, 0 10)'"
+        -> "POINT (0 5)",
+      "'POLYGON((0 0, 0 5, 5 5, 5 0, 0 0))'"
+        -> "POINT (2.5 2.5)",
+      "'LINESTRING(0 5 1, 0 0 1, 0 10 2)'"
+        -> "POINT Z(0 0 1)"
+    )
+
+        for((inputGeom, expectedGeom) <- geomTestCases1) {
+      var df = sparkSession.sql(s"select ST_AsText(ST_PointOnSurface(ST_GeomFromText($inputGeom)))")
+      var result = df.collect()
+      assert(result.head.get(0).asInstanceOf[String]==expectedGeom)
+    }
+
+    /* ST_AsEWKT Has not been implemented yet
+
+    val geomTestCases2 = Map(
+      "'LINESTRING(0 5 1, 0 0 1, 0 10 2)'"
+        -> "POINT (0 0 1)"
+    )
+
+    for((inputGeom, expectedGeom) <- geomTestCases2) {
+      var df = sparkSession.sql(s"select ST_AsEWKT(ST_PointOnSurface(ST_GeomFromEWKT($inputGeom)))")
+      var result = df.collect()
+      assert(result.head.get(0).asInstanceOf[String]==expectedGeom)
+    }
+    */
+  }
+
+  it ("Should pass ST_Reverse") {
+    val geomTestCases = Map(
+      "'POLYGON((-1 0 0, 1 0 0, 0 0 1, 0 1 0, -1 0 0))'"
+        -> "POLYGON Z((-1 0 0, 0 1 0, 0 0 1, 1 0 0, -1 0 0))",
+      "'LINESTRING(0 0, 1 2, 2 4, 3 6)'"
+        -> "LINESTRING (3 6, 2 4, 1 2, 0 0)",
+      "'POINT(1 2)'"
+        -> "POINT (1 2)",
+      "'MULTIPOINT((10 40 66), (40 30 77), (20 20 88), (30 10 99))'"
+        -> "MULTIPOINT Z((10 40 66), (40 30 77), (20 20 88), (30 10 99))",
+      """'MULTIPOLYGON(((30 20 11, 45 40 11, 10 40 11, 30 20 11)),
+      |((15 5 11, 40 10 11, 10 20 11, 5 10 11, 15 5 11)))'""".stripMargin.replaceAll("\n", " ")
+        -> """MULTIPOLYGON Z(((30 20 11, 10 40 11, 45 40 11, 30 20 11)),
+          |((15 5 11, 5 10 11, 10 20 11, 40 10 11, 15 5 11)))""".stripMargin.replaceAll("\n", " "),
+      """'MULTILINESTRING((10 10 11, 20 20 11, 10 40 11),
+        |(40 40 11, 30 30 11, 40 20 11, 30 10 11))'""".stripMargin.replaceAll("\n", " ")
+        -> """MULTILINESTRING Z((10 40 11, 20 20 11, 10 10 11),
+          |(30 10 11, 40 20 11, 30 30 11, 40 40 11))""".stripMargin.replaceAll("\n", " "),
+      """'MULTIPOLYGON(((40 40 11, 20 45 11, 45 30 11, 40 40 11)),
+      |((20 35 11, 10 30 11, 10 10 11, 30 5 11, 45 20 11, 20 35 11),
+      |(30 20 11, 20 15 11, 20 25 11, 30 20 11)))'""".stripMargin.replaceAll("\n", " ")
+        -> """MULTIPOLYGON Z(((40 40 11, 45 30 11, 20 45 11, 40 40 11)),
+          |((20 35 11, 45 20 11, 30 5 11, 10 10 11, 10 30 11, 20 35 11),
+          |(30 20 11, 20 25 11, 20 15 11, 30 20 11)))""".stripMargin.replaceAll("\n", " "),
+      """'POLYGON((0 0 11, 0 5 11, 5 5 11, 5 0 11, 0 0 11),
+      |(1 1 11, 2 1 11, 2 2 11, 1 2 11, 1 1 11))'""".stripMargin.replaceAll("\n", " ")
+        -> """POLYGON Z((0 0 11, 5 0 11, 5 5 11, 0 5 11, 0 0 11),
+          |(1 1 11, 1 2 11, 2 2 11, 2 1 11, 1 1 11))""".stripMargin.replaceAll("\n", " ")
+    )
+    for((inputGeom, expectedGeom) <- geomTestCases) {
+      var df = sparkSession.sql(s"select ST_AsText(ST_Reverse(ST_GeomFromText($inputGeom)))")
+      var result = df.collect()
+      assert(result.head.get(0).asInstanceOf[String]==expectedGeom)
+    }
+  }
+
+  it("Should pass ST_PointN") {
+
+    Given("Some different types of geometries in a DF")
+
+    val sampleLineString = "LINESTRING(0 0, 1 2, 2 4, 3 6)"
+    val testData = Seq(
+      (sampleLineString, 1),
+      (sampleLineString, 2),
+      (sampleLineString, -1),
+      (sampleLineString, -2),
+      (sampleLineString, 3),
+      (sampleLineString, 4),
+      (sampleLineString, 5),
+      (sampleLineString, -5),
+      ("POLYGON((-1 0 0, 1 0 0, 0 0 1, 0 1 0, -1 0 0))", 2),
+      ("POINT(1 2)", 1)
+    ).toDF("Geometry", "N")
+
+    When("Using ST_PointN for getting the nth point in linestring type of Geometries")
+
+    val testDF = testData.selectExpr("ST_PointN(ST_GeomFromText(Geometry), N) as geom")
+
+    Then("Result should match the list of nth points")
+
+    testDF.selectExpr("ST_AsText(geom)")
+      .as[String].collect() should contain theSameElementsAs
+      List(
+        "POINT (0 0)", "POINT (1 2)", "POINT (3 6)",
+        "POINT (2 4)", "POINT (2 4)", "POINT (3 6)",
+        null, null, null, null
+      )
+  }
+
+  it ("Should pass ST_AsEWKT") {
+    var df = sparkSession.sql("SELECT ST_SetSrid(ST_GeomFromWKT('POLYGON((0 0,0 1,1 1,1 0,0 0))'), 4326) as point")
+    df.createOrReplaceTempView("table")
+    df = sparkSession.sql("SELECT ST_AsEWKT(point) from table")
+    val s = "SRID=4326;POLYGON ((0 0, 0 1, 1 1, 1 0, 0 0))"
+    assert(df.first().get(0).asInstanceOf[String] == s)
+  }
+
+  it ("Should pass ST_Force_2D") {
+    val geomTestCases1 = Map(
+      "'POINT(0 5)'"
+        -> "POINT (0 5)",
+      "'POLYGON((0 0 2, 0 5 2, 5 0 2, 0 0 2), (1 1 2, 3 1 2, 1 3 2, 1 1 2))'"
+        -> "POLYGON ((0 0, 0 5, 5 0, 0 0), (1 1, 3 1, 1 3, 1 1))",
+      "'LINESTRING(0 5 1, 0 0 1, 0 10 2)'"
+        -> "LINESTRING (0 5, 0 0, 0 10)"
+    )
+
+    for ((inputGeom, expectedGeom) <- geomTestCases1) {
+      var df = sparkSession.sql(s"select ST_AsText(ST_Force_2D(ST_GeomFromText($inputGeom)))")
+      var result = df.collect()
+      assert(result.head.get(0).asInstanceOf[String] == expectedGeom)
+    }
+  }
+
+  it ("Should pass ST_IsEmpty") {
+    var df = sparkSession.sql("SELECT ST_SetSrid(ST_GeomFromWKT('POLYGON((0 0,0 1,1 1,1 0,0 0))'), 4326) as point")
+    df.createOrReplaceTempView("table")
+    df = sparkSession.sql("SELECT ST_IsEmpty(point) from table")
+    val s = false
+    assert(df.first().get(0).asInstanceOf[Boolean] == s)
+  }
+
+  it("Passed ST_XMax") {
+    var test = sparkSession.sql("SELECT ST_XMax(ST_GeomFromWKT('POLYGON ((-1 -11, 0 10, 1 11, 2 12, -1 -11))'))")
+    assert(test.take(1)(0).get(0).asInstanceOf[Double] == 2.0)
+
+  }
+
+  it("Passed ST_XMin") {
+    var test = sparkSession.sql("SELECT ST_XMin(ST_GeomFromWKT('POLYGON ((-1 -11, 0 10, 1 11, 2 12, -1 -11))'))")
+    assert(test.take(1)(0).get(0).asInstanceOf[Double] == -1.0)
+
+  }
+
+  it ("Should pass ST_BuildArea") {
+    val geomTestCases = Map(
+      "'MULTILINESTRING((0 0, 10 0, 10 10, 0 10, 0 0),(10 10, 20 10, 20 20, 10 20, 10 10))'"
+        -> "MULTIPOLYGON (((0 0, 0 10, 10 10, 10 0, 0 0)), ((10 10, 10 20, 20 20, 20 10, 10 10)))",
+      "'MULTILINESTRING((0 0, 10 0, 10 10, 0 10, 0 0),(10 10, 20 10, 20 0, 10 0, 10 10))'"
+        -> "POLYGON ((0 0, 0 10, 10 10, 20 10, 20 0, 10 0, 0 0))",
+      "'MULTILINESTRING((0 0, 20 0, 20 20, 0 20, 0 0),(2 2, 18 2, 18 18, 2 18, 2 2))'"
+        -> "POLYGON ((0 0, 0 20, 20 20, 20 0, 0 0), (2 2, 18 2, 18 18, 2 18, 2 2))",
+      "'MULTILINESTRING((0 0, 20 0, 20 20, 0 20, 0 0), (2 2, 18 2, 18 18, 2 18, 2 2), (8 8, 8 12, 12 12, 12 8, 8 8))'"
+        -> "MULTIPOLYGON (((0 0, 0 20, 20 20, 20 0, 0 0), (2 2, 18 2, 18 18, 2 18, 2 2)), ((8 8, 8 12, 12 12, 12 8, 8 8)))",
+      """'MULTILINESTRING((0 0, 20 0, 20 20, 0 20, 0 0),(2 2, 18 2, 18 18, 2 18, 2 2),
+        |(8 8, 8 9, 8 10, 8 11, 8 12, 9 12, 10 12, 11 12, 12 12, 12 11, 12 10, 12 9, 12 8, 11 8, 10 8, 9 8, 8 8))'""".stripMargin.replaceAll("\n", " ")
+        -> """MULTIPOLYGON (((0 0, 0 20, 20 20, 20 0, 0 0), (2 2, 18 2, 18 18, 2 18, 2 2)),
+             |((8 8, 8 9, 8 10, 8 11, 8 12, 9 12, 10 12, 11 12, 12 12, 12 11, 12 10, 12 9, 12 8, 11 8, 10 8, 9 8, 8 8)))""".stripMargin.replaceAll("\n", " "),
+      "'MULTILINESTRING((0 0, 20 0, 20 20, 0 20, 0 0),(2 2, 18 2, 18 18, 2 18, 2 2),(8 8, 8 12, 12 12, 12 8, 8 8),(10 8, 10 12))'"
+        -> "MULTIPOLYGON (((0 0, 0 20, 20 20, 20 0, 0 0), (2 2, 18 2, 18 18, 2 18, 2 2)), ((8 8, 8 12, 12 12, 12 8, 8 8)))",
+      "'MULTILINESTRING((0 0, 20 0, 20 20, 0 20, 0 0),(2 2, 18 2, 18 18, 2 18, 2 2),(10 2, 10 18))'"
+        -> "POLYGON ((0 0, 0 20, 20 20, 20 0, 0 0), (2 2, 18 2, 18 18, 2 18, 2 2))",
+      """'MULTILINESTRING( (0 0, 70 0, 70 70, 0 70, 0 0), (10 10, 10 60, 40 60, 40 10, 10 10),
+        |(20 20, 20 30, 30 30, 30 20, 20 20), (20 30, 30 30, 30 50, 20 50, 20 30), (50 20, 60 20, 60 40, 50 40, 50 20),
+        |(50 40, 60 40, 60 60, 50 60, 50 40), (80 0, 110 0, 110 70, 80 70, 80 0), (90 60, 100 60, 100 50, 90 50, 90 60))'""".stripMargin.replaceAll("\n", " ")
+        -> """MULTIPOLYGON (((0 0, 0 70, 70 70, 70 0, 0 0), (10 10, 40 10, 40 60, 10 60, 10 10), (50 20, 60 20, 60 40, 60 60, 50 60, 50 40, 50 20)),
+          |((20 20, 20 30, 20 50, 30 50, 30 30, 30 20, 20 20)),
+          |((80 0, 80 70, 110 70, 110 0, 80 0), (90 50, 100 50, 100 60, 90 60, 90 50)))""".stripMargin.replaceAll("\n", " ")
+    )
+
+    for ((inputGeom, expectedGeom) <- geomTestCases) {
+      val df = sparkSession.sql(s"select ST_AsText(ST_BuildArea(ST_GeomFromText($inputGeom)))")
+      val result = df.collect()
+      assert(result.head.get(0).asInstanceOf[String] == expectedGeom)
+    }
+  }
+
   it("handles nulls") {
     var functionDf: DataFrame = null
     functionDf = sparkSession.sql("select ST_Distance(null, null)")
@@ -1391,5 +1565,28 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
     assert(functionDf.first().get(0) == null)
     functionDf = sparkSession.sql("select ST_Union(null, null)")
     assert(functionDf.first().get(0) == null)
+    functionDf = sparkSession.sql("select ST_PointOnSurface(null)")
+    assert(functionDf.first().get(0) == null)
+    functionDf = sparkSession.sql("select ST_Reverse(null)")
+    assert(functionDf.first().get(0) == null)
+    functionDf = sparkSession.sql("select ST_AsEWKT(null)")
+    assert(functionDf.first().get(0) == null)
+    functionDf = sparkSession.sql("select ST_Force_2D(null)")
+    assert(functionDf.first().get(0) == null)
+    functionDf = sparkSession.sql("select ST_BuildArea(null)")
+    assert(functionDf.first().get(0) == null)
   }
-}
+
+  it ("Should pass St_CollectionExtract") {
+    var df = sparkSession.sql("SELECT ST_GeomFromText('GEOMETRYCOLLECTION(POINT(40 10), LINESTRING(0 5, 0 10), POLYGON((0 0, 0 5, 5 5, 5 0, 0 0)))') as geom")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom))").collect().head.get(0) == "MULTIPOLYGON (((0 0, 0 5, 5 5, 5 0, 0 0)))")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom, 3))").collect().head.get(0) == "MULTIPOLYGON (((0 0, 0 5, 5 5, 5 0, 0 0)))")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom, 1))").collect().head.get(0) == "MULTIPOINT ((40 10))")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom, 2))").collect().head.get(0) == "MULTILINESTRING ((0 5, 0 10))")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom, 2))").collect().head.get(0) == "MULTILINESTRING ((0 5, 0 10))")
+    df = sparkSession.sql("SELECT ST_GeomFromText('GEOMETRYCOLLECTION (POINT (40 10), POINT (40 10))') as geom")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom, 1))").collect().head.get(0) == "MULTIPOINT ((40 10), (40 10))")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom, 2))").collect().head.get(0) == "MULTILINESTRING EMPTY")
+    assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom))").collect().head.get(0) == "MULTIPOINT ((40 10), (40 10))")
+  }
+}
\ No newline at end of file
